"use strict";
var util = require("util");

var AnalyticsCommand = (function () {
    function AnalyticsCommand($analyticsService, $logger, $errors) {
        this.$analyticsService = $analyticsService;
        this.$logger = $logger;
        this.$errors = $errors;
        this.allowedParameters = [new AnalyticsCommandParameter(this.$errors)];
    }
    AnalyticsCommand.prototype.execute = function (args) {
        var _this = this;
        return (function () {
            var arg = args[0];
            switch (arg) {
                case "enable":
                    _this.$analyticsService.setAnalyticsStatus(true).wait();
                    _this.$logger.info("Feature usage tracking is now enabled.");
                    break;
                case "disable":
                    _this.$analyticsService.disableAnalytics().wait();
                    _this.$logger.info("Feature usage tracking is now disabled.");
                    break;
                case "status":
                case undefined:
                    _this.$logger.out(_this.$analyticsService.getStatusMessage().wait());
                    break;
                default:
                    _this.$errors.fail(util.format("The value '%s' is not valid. Valid values are 'enable', 'disable' and 'status'.", arg));
            }
        }).future()();
    };
    return AnalyticsCommand;
})();
exports.AnalyticsCommand = AnalyticsCommand;
$injector.registerCommand("feature-usage-tracking", AnalyticsCommand);

var AnalyticsCommandParameter = (function () {
    function AnalyticsCommandParameter($errors) {
        this.$errors = $errors;
        this.mandatory = false;
    }
    AnalyticsCommandParameter.prototype.validate = function (validationValue) {
        var _this = this;
        return (function () {
            switch (validationValue) {
                case "enable":
                case "disable":
                case "status":
                case undefined:
                    return true;
                default:
                    _this.$errors.fail(util.format("The value '%s' is not valid. Valid values are 'enable', 'disable' and 'status'.", validationValue));
            }
        }).future()();
    };
    return AnalyticsCommandParameter;
})();
exports.AnalyticsCommandParameter = AnalyticsCommandParameter;
