"use strict";
var path = require("path");
var os = require("os");

var ProjectData = (function () {
    function ProjectData($fs, $errors, $projectHelper, $staticConfig) {
        this.$fs = $fs;
        this.$errors = $errors;
        this.$projectHelper = $projectHelper;
        this.$staticConfig = $staticConfig;
        this.initializeProjectData().wait();
    }
    ProjectData.prototype.initializeProjectData = function () {
        var _this = this;
        return (function () {
            var projectDir = _this.$projectHelper.projectDir;

            if (projectDir) {
                _this.projectDir = projectDir;
                _this.projectName = path.basename(projectDir);
                _this.platformsDir = path.join(projectDir, "platforms");
                _this.projectFilePath = path.join(projectDir, _this.$staticConfig.PROJECT_FILE_NAME);

                if (_this.$fs.exists(_this.projectFilePath).wait()) {
                    try  {
                        var fileContent = _this.$fs.readJson(_this.projectFilePath).wait();
                        _this.projectId = fileContent.id;
                    } catch (err) {
                        _this.$errors.fail({
                            formatStr: "The project file %s is corrupted." + os.EOL + "Consider restoring an earlier version from your source control or backup." + os.EOL + "Additional technical info: %s",
                            suppressCommandHelp: true }, _this.projectFilePath, err.toString());
                    }
                }
            } else {
                _this.$errors.fail("No project found at or above '%s' and neither was a --path specified.", process.cwd());
            }
        }).future()();
    };
    return ProjectData;
})();
exports.ProjectData = ProjectData;
$injector.register("projectData", ProjectData);
